# django-clite
from django_clite.cli.commands.base_helper import BaseHelper
from django_clite.cli.commands.creator.main import create
from django_clite.cli.commands.destroyer.main import destroy
from django_clite.cli.commands.generator.main import generate
from django_clite.cli.commands.run.main import run



from jinja2 import Template

env_template = Template("""# Example variables that could be in an environment file.
# Commit only the .env-example file, not the .env file. Use the example file to illustrate
# what variables your environment uses, but be sure to not commit this file with any sensitive information.

DEBUG=True

ADMINS=# i.e (('Admin', 'admin@example.com'),)


SECRET_KEY=


SERVER_EMAIL=# i.e no-reply@example.com
EMAIL_HOST=
EMAIL_HOST_USER=
EMAIL_HOST_PASSWORD=
EMAIL_RELAY_DOMAIN=# relay.example.com


SENDGRID_API_KEY=


STRIPE_LIVE_MODE=False
STRIPE_LIVE_PUBLIC_KEY=
STRIPE_LIVE_SECRET_KEY=
STRIPE_TEST_SECRET_KEY=
STRIPE_TEST_PUBLISHABLE_KEY=
STRIPE_WEBHOOK_SIGNING_SECRET=


OTP_ISSUER='Organization Name'
TWILIO_SID=
TWILIO_TOKEN=
TWILIO_CALLER_ID=


AWS_BUCKET_NAME=
AWS_LOCATION=
AWS_STATIC_LOCATION=
AWS_MEDIA_LOCATION=
AWS_PRIVATE_MEDIA_LOCATION=
AWS_REGION_NAME=
AWS_ORIGIN=
AWS_ENDPOINT=
AWS_ENDPOINT_URL=
AWS_CUSTOM_DOMAIN=
AWS_ACCESS_KEY_ID=
AWS_SECRET_ACCESS_KEY=


CLOUDINARY_URL=


REDIS_URL=# i.e. redis://[:password]@127.0.0.1:6379


DOCKER_DB_NAME=
DOCKER_DB_USER=
DOCKER_DB_PASSWORD=

LOCAL_DB_NAME=
LOCAL_DB_USER=
LOCAL_DB_PASSWORD=
LOCAL_DB_HOST=localhost

DOKKU_LETSENCRYPT_EMAIL=#required

SENTRY_DSN=


GITHUB_WEBHOOKS_VERIFICATION_KEY=
""")



# django-clite
from __future__ import unicode_literals
import click
from .cli import commands
from .cli import templates


class AliasedGroup(click.Group):
    """
    Adds support for abbreviated commands
    """
    def get_command(self, ctx, cmd_name):
        rv = click.Group.get_command(self, ctx, cmd_name)
        if rv is not None:
            return rv
        matches = [x for x in self.list_commands(ctx)
                   if x.startswith(cmd_name)]
        if not matches:
            return None
        elif len(matches) == 1:
            return click.Group.get_command(self, ctx, matches[0])
        ctx.fail('Too many matches: %s' % ', '.join(sorted(matches)))



from django_clite.cli import log_success
from django_clite.cli.commands.base_helper import BaseHelper
from django_clite.cli.templates.form import (
    model_form_template,
    model_form_import_template
)


class FormHelper(BaseHelper):

    def create(self, **kwargs):

        model = self.check_noun(kwargs['model'])
        path = kwargs['path']

        filename = f"{model.lower()}.py"

        self.parse_and_create(
            filename=filename,
            model=model,
            path=path,
            template=model_form_template,
            dry=kwargs['dry']
        )

        self.add_import(
            model=model,
            template=model_form_import_template,
            path=path,
            dry=kwargs['dry']
        )

        log_success('Successfully created form.')

    def delete(self, **kwargs):
        model = self.check_noun(kwargs['model'])

        filename = f"{model.lower()}.py"

        template = model_form_import_template

        if self.destroy(filename=filename, **kwargs):

            self.remove_import(template=template, **kwargs)

            log_success('Successfully deleted form.')

    @classmethod
    def create_auth_user(cls, **kwargs):
        cls.parse_and_create(
            filename=kwargs['filename'],
            model=kwargs['model'],
            path=kwargs['path'],
            template=model_form_template
        )

        cls.add_import(**kwargs, template=model_form_import_template)

# end class



from jinja2 import Template


dockerfile_template = Template("""FROM python:3.6.5

# Environment variables
ENV PYTHONUNBUFFERED 1
RUN apt-get update

# Install some necessary dependencies.
RUN apt-get install -y swig libssl-dev dpkg-dev netcat

RUN pip install -U pip
ADD requirements.txt /code/
RUN pip install -Ur /code/requirements.txt

# Set working directory
WORKDIR /code/

# Copy requirements to working directory
COPY ./requirements.txt .

# Copy project files
COPY . /code/
""")


docker_compose_template = Template("""version: '3'

services:
  web:
    container_name: "{{ project.lower() }}-web"
    labels:
        com.{{ project.lower() }}.web.description = "{{ project }}: Web Application"
    build: ./application
    volumes:
        - ./application/:code/
        - static_volume:/var/www/staticfiles
        - media_volume:/var/www/mediafiles
    ports:
        - "8093:8000"
    networks:
        - {{ project }}_network 
    env_file:
        - .env
    depends_on:
        - db
    command: gunicorn {{ project }}.wsgi:application --bind 0.0.0.0:8000 --workers 3
  
  db:
    image: postgres:10.5-alpine
    labels:
        com.{{ project.lower() }}.db.description: "{{ project.lower() }}: Database service"
    volumes:
      - postgres_data:/var/lib/postgresql/data/
    networks:
        - {{ project.lower() }}_network
    restart: always
  
  nginx:
    container_name: "{{ project.lower() }}-nginx"
    labels:
        com.{{ project.lower() }}.nginx.description: "{{ project.lower() }}: Proxy Server"
    image: nginx:1.15.0-alpine
    volumes:
      - static_volume:/var/www/staticfiles
      - media_volume:/var/www/mediafiles
    ports:
      - 1337:80
    depends_on:
      - web
  
  redis:
    restart: always
    image: redis:latest
    expose:
      - "6379"

volumes:
  postgres_data:
  static_volume:
  media_volume:
""")



import os
import subprocess


DEFAULT_MAKE_MIGRATIONS_COMMAND = ['python3', 'manage.py', 'makemigrations']

DEFAULT_MIGRATE_COMMAND = ['python3', 'manage.py', 'migrate']


class MigrationHelper(object):

    @classmethod
    def make_migrations(cls, path, app, options):
        os.chdir(path)

        if app is not None:
            DEFAULT_MAKE_MIGRATIONS_COMMAND.append(app)

        if options:
            for option in options:
                DEFAULT_MAKE_MIGRATIONS_COMMAND.append(option)

        subprocess.check_output(DEFAULT_MAKE_MIGRATIONS_COMMAND)

    @classmethod
    def migrate(cls, path, app, options):
        os.chdir(path)

        if app is not None:
            DEFAULT_MIGRATE_COMMAND.append(app)

        if options:
            for option in options:
                DEFAULT_MIGRATE_COMMAND.append(option)

        subprocess.check_output(DEFAULT_MIGRATE_COMMAND)

    @classmethod
    def run(cls, path, app, general, direction, options):
        if general:
            if direction:
                cls.migrate(path, app=None, options=options)
            else:
                cls.make_migrations(path, app=None, options=options)
                cls.migrate_migrations(path, app=None, options=options)
        elif app:
            cls.make_migrations(path, app, options)
            cls.migrate(path, app, options=options)



import click
import os
import subprocess
from django.core.exceptions import ImproperlyConfigured

DEFAULT_MIGRATIONS_COMMAND = ['python3', 'manage.py', 'makemigrations']

DEFAULT_LOAD_COMMAND = ['python3', 'manage.py', 'loaddata']

DEFAULT_COLLECT_COMMAND = ['python3', 'manage.py', 'collectstatic', '--no-input']


class BuildHelper(object):

    @classmethod
    def start(cls, path, options=None):
        """
        Runs migrations, loads fixtures, and collects static files.
        """
        os.chdir(path)

        if options is not None:
            click.echo('Options passed...')

        try:
            # res = subprocess.check_call(DEFAULT_MIGRATIONS_COMMAND)
            # res = subprocess.call(DEFAULT_LOAD_COMMAND)
            res = subprocess.check_output(DEFAULT_COLLECT_COMMAND)
        except subprocess.CalledProcessError:
            click.Abort()



import click
import fileinput
import inflection
import os
from django_clite.cli import log_success, sanitized_string, log_info
from django_clite.cli.commands.base_helper import BaseHelper
from django_clite.cli.templates.model import (
    auth_user_model_template,
    model_field_template,
    model_import_template,
    model_template,
    sql_view_template
)


DEFAULT_NOT_IN_SCOPE_WARNING = """Cannot find model {} in {} scope. Want to create model?"""

# Abbreviation, field name, default value
DEFAULT_MODEL_FIELDS = {
    'belongsto': "ForeignKey",
    'big': 'BigIntegerField',
    'bigint': 'BigIntegerField',
    'bool': 'BooleanField',
    'boolean': 'BooleanField',
    'char': 'CharField',
    'date': 'DateField',
    'datetime': 'DateTimeField',
    'dec': 'DecimalField',
    'decimal': 'DecimalField',
    'duration': 'DurationField',
    'email': 'EmailField',
    'file': "FileField",
    'filepath': 'FilePathField',
    'fk': "ForeignKey",
    'foreignkey': "ForeignKey",
    'float': 'FloatField',
    'hasone': "OneToOneField",
    'hasmany': 'ManyToManyField',
    'image': "ImageField",
    'int': 'IntegerField',
    'integer': 'IntegerField',
    'ipaddress': 'GenericIPAddressField',
    'many': 'ManyToManyField',
    'manytomany': 'ManyToManyField',
    'one': "OneToOneField",
    'onetoone': "OneToOneField",
    'photo': "ImageField",
    'slug': 'SlugField',
    'string': 'TextField',
    'text': 'TextField',
    'time': 'TimeField',
    'url': 'URLField',
    'uuid': 'UUIDField',
}

# Abbreviation, field name, default value
DEFAULT_MODEL_OPTIONS = {
    'BigIntegerField': '',
    'BooleanField': 'default=False',
    'CharField': 'max_length=100',
    'DateField': 'auto_now=True',
    'DateTimeField': 'auto_now=True',
    'DecimalField': '',
    'DurationField': '',
    'EmailField': '',
    'GenericIPAddressField': '',
    'FileField': "blank=True, upload_to='uploads/{}/'",
    'FilePathField': '',
    'FloatField': '',
    'ForeignKey': "{}, related_name='{}', on_delete=models.PROTECT",
    'ImageField': "blank=True, upload_to='uploads/{}/'",
    'IntegerField': '',
    'ManyToManyField': '{}, blank=True',
    'OneToOneField': "{}, related_name='{}', on_delete=models.CASCADE",
    'SlugField': 'unique=True',
    'TextField': 'blank=True',
    'TimeField': 'auto_now=True',
    'URLlField': '',
    'UUIDField': 'default=uuid.uuid4, editable=False',
}

DEFAULT_SPECIAL_INHERITABLE_TYPES = {
    'abstract-user': 'django.contrib.auth.models',
    'abstract-base-user': 'django.contrib.auth.models',
    'base-user': 'django.contrib.auth.models',
    'user': 'django.contrib.auth.models',
}


class ModelHelper(BaseHelper):

    # List of fields to be rendered in model template
    fields_list = []

    # List models referred to in ForeignKey, OneToOne, or ManyToMany fields.
    dependencies_list = []

    # List of imports to be rendered in model template
    imports_list = []

    # List of special imports to be rendered in model template
    special_import = []

    @classmethod
    def append_import(cls, value):
        if value not in cls.imports_list:
            cls.imports_list.append(value)

    @classmethod
    def append_special_import(cls, value):
        if value in DEFAULT_SPECIAL_INHERITABLE_TYPES:
            statement = DEFAULT_SPECIAL_INHERITABLE_TYPES[value]
            value = inflection.camelize(inflection.underscore(value))
            cls.special_import = (statement, value)
            return True
        else:
            model = inflection.camelize(inflection.underscore(value))
            value = inflection.underscore(value)
            cls.special_import = (f'.{value}', model)
        return False

    def create(self, **kwargs):
        model = self.check_noun(kwargs['model'])

        base_model = self.check_noun(kwargs['inherits']) if kwargs['inherits'] else None

        # Check if model is in the `special` category before
        # attempting to create a module for it if one is not found.
        if base_model is not None:
            if not self.append_special_import(base_model):
                self.find_resource_in_scope(base_model)

        app_name = self.get_app_name()

        path = kwargs['path']

        filename = f"{model}.py"

        db_table_name = f"{app_name}_{inflection.pluralize(model)}"

        template = model_template

        if kwargs['fields'] is not None:
            self.parse_fields(**kwargs)

        if kwargs['view']:
            template = sql_view_template

        self.parse_and_create(
            model=model,
            abstract=kwargs['abstract'],
            base=self.special_import,
            fields=self.fields_list,
            imports=self.imports_list,
            db_table=db_table_name,
            template=template,
            filename=filename,
            path=path,
            dry=kwargs['dry']
        )

        self.add_import(**kwargs, template=model_import_template)

        log_success("Successfully created model.")

        # Ensure related models are created
        self.handle_dependencies(**kwargs, app_name=app_name)

        return

    @classmethod
    def create_auth_user(cls, **kwargs):

        kwargs['model'] = 'User'

        kwargs['filename'] = 'user.py'

        kwargs['path'] = f"{kwargs['path']}/models/"

        cls.parse_and_create(
            model=kwargs['model'],
            filename=kwargs['filename'],
            project_name=kwargs['project'],
            template=auth_user_model_template,
            path=kwargs['path']
        )

        cls.add_import(**kwargs, template=model_import_template)

        log_success("Successfully created model.")

    def delete(self, **kwargs):
        model = self.check_noun(kwargs['model'])

        filename = f"{model.lower()}.py"

        template = model_import_template

        if self.destroy(filename=filename, **kwargs):

            self.remove_import(template=template, **kwargs)

            log_success('Successfully deleted model.')

    @classmethod
    def find_resource_in_scope(cls, model):
        """
        Searches the current directory for the specified model
        either by filename or import statement in __init__.py

        If the model is not found in the current scope, a warning is displayed
        asking the user whether the model should be created.
        """
        path = f"{os.getcwd()}/models"

        init = f"{path}/__init__.py"

        app_name = cls.get_app_name()

        filename = model.lower()

        if not model.endswith('.py'):
            filename += '.py'

        if filename in os.listdir(path):
            return

        for line in fileinput.input(init):
            if model.capitalize() in line:
                fileinput.close()
                return
        fileinput.close()

        if click.confirm(DEFAULT_NOT_IN_SCOPE_WARNING.format(model.capitalize(), app_name)):
            cls.dependencies_list.append(model.capitalize())

    @classmethod
    def get_app_name(cls):
        """
        Searches current directory for apps.py in order to
        retrieve the application name from it.
        """
        try:
            for line in fileinput.input('apps.py'):
                if "name = " in line:
                    fileinput.close()
                    return line.split(" = ")[1].lstrip().replace("\n", "").replace("'", "")
            fileinput.close()
        except FileNotFoundError:
            return "app"
        return "app"

    def handle_dependencies(self, **kwargs):

        if len(self.dependencies_list) < 1:
            return False

        for model in self.dependencies_list:
            app_name = kwargs['app_name']

            filename = f'{model.lower()}.py'

            db_table_name = f"{app_name}_{inflection.pluralize(model)}"

            self.parse_and_create(
                model=model,
                abstract=False,
                fields=[],
                imports=[],
                db_table=db_table_name,
                template=model_template,
                filename=filename,
                path=kwargs['path'],
                dry=kwargs['dry']
            )

            self.add_import(
                template=model_import_template,
                model=model,
                path=kwargs['path'],
                dry=kwargs['dry']
            )

        return True

    @classmethod
    def handle_tokens(cls, field):
        """
        Should parse input attribute into individual tokens.
        Expect input to be of the form (char:name, text:bio, image:artwork)
        """

        # i.e char:first_name
        parsed_token = field.split(":")

        # i.e char
        parsed_token_type = parsed_token[0]

        # i.e first_name
        parsed_token_name = parsed_token[1]

        # Ensure token exists and has minimum required length of 2
        for part in parsed_token:
            if len(part) < 2:
                return None

        # Check if parsed_token_type is a relationship field
        for abbreviation, field in DEFAULT_MODEL_FIELDS.items():
            if parsed_token_type.lower() == abbreviation.lower():
                return sanitized_string(parsed_token_name), DEFAULT_MODEL_FIELDS[abbreviation]

        return None

    @classmethod
    def handle_field(cls, model, field_name, field_type):
        """
        Determines what options are present with each model field.
        """

        if DEFAULT_MODEL_OPTIONS[field_type] is not None:
            if field_type == "ForeignKey":
                options = DEFAULT_MODEL_OPTIONS[field_type].format(
                    field_name.capitalize(),
                    inflection.pluralize(model.lower())
                )
                cls.find_resource_in_scope(field_name)
                cls.append_import(field_name)

            elif field_type == "OneToOneField":
                options = DEFAULT_MODEL_OPTIONS[field_type].format(
                      field_name.capitalize(),
                      inflection.singularize(model)
                  )
                cls.find_resource_in_scope(field_name)
                cls.append_import(field_name)

            elif field_type == "ManyToManyField":
                options = DEFAULT_MODEL_OPTIONS[field_type].format(
                        field_name.capitalize()
                )
                cls.find_resource_in_scope(field_name)
                cls.append_import(field_name)

            elif field_type == "FileField" or "ImageField":
                options = DEFAULT_MODEL_OPTIONS[field_type].format(inflection.pluralize(field_name))

            else:
                options = DEFAULT_MODEL_OPTIONS[field_type]

            return model_field_template.render(
                name=field_name,
                type=field_type,
                options=options
            )

    @classmethod
    def parse_fields(cls, **kwargs):
        """
        Parses and saves field data in fields_list for future reference
        """
        try:
            fields = kwargs['fields']
        except KeyError:
            return

        for field in fields:
            try:
                field_name, field_type = cls.handle_tokens(field)

                if field_type is not None and field_name is not None:
                    field_content = cls.handle_field(
                        field_type=field_type,
                        field_name=field_name,
                        model=kwargs['model']
                    )

                    cls.fields_list.append(field_content)

            except TypeError:
                return



import click
import os
from django_clite.cli import find_management_file, get_project_name, log_error
from .helpers import (
    BuildHelper,
    DockerHelper,
    EnvironmentHelper,
    FixtureHelper,
    MigrationHelper,
    ServerHelper
)

DEFAULT_MISSING_ARGS_ERROR = 'Missing arguments: {}.'
DEFAULT_TOO_MANY_ARGS_ERROR = 'Too many arguments passed.'


def not_an_app_directory_warning():
    if not ('apps.py' in os.listdir('.')):
        log_error("Not inside an app directory")
        raise click.Abort


def not_in_project_warning():
    path, management, code, file = find_management_file(os.getcwd())
    if not management:
        log_error('Cannot find manage.py in directory')
        raise click.Abort


@click.group()
@click.pass_context
def run(ctx):
    """
    Run maintenance, development, and deployment scripts.
    """

    not_in_project_warning()

    ctx.ensure_object(dict)

    p, m, c, f = find_management_file(os.getcwd())

    ctx.obj['path'] = p
    ctx.obj['management'] = m
    ctx.obj['code'] = c
    ctx.obj['file'] = f
    ctx.obj['project_name'] = get_project_name(f)


# @run.command()
@click.pass_context
def build(ctx):
    """
    Run migrations and collect static files.
    """

    path = ctx.obj['management']
    BuildHelper.start(path)


# @run.command()
@click.pass_context
def deploy(ctx):
    """
    Push your project to your deployment server.

    This will prepare your project for deployment and run some checks for you. For example, the CLI will check if
    DEBUG or SECRET_KEY are properly configured. It will check if you have pending migrations and also collect
    staticfiles to your specified staticfiles location.

    First define your deployment upstream so the CLI can push your code to it. Once that is defined,
    in .config/upstream, the CLI will use the upstream server information to deploy your code.

    Supported deployment upstreams:
    - Dokku

    \b
        Define your upstream as DOKKU and add a DOKKU_HOST in your upstream configuration.
        The host needs to be an IP or fully qualified domain name (FQDN) for your upstream server.

    - Bitbucket, Github, and Gitlab

    \b
        Define your upstream as GIT and if your .git repository has an `origin` remote,
        the CLI will push your code to that server.
    """
    pass


# @run.command()
@click.pass_context
def docker(ctx):
    """
    Run project from within a Docker container.
    """
    pass


@run.command()
@click.pass_context
@click.option('-f', '--filepath', type=click.Path(exists=True), required=False, help='Path to environment file.')
def export_env(ctx, filepath):
    """
    Export environment variables for for example file and dokku config.
    For example environment file, all values are striped out, only keys are exported.

    \b
    In .env-dokku file:
    dokku config:set --no-restart PROJECT_NAME VARIABLE1=value1
    dokku config:set --no-restart PROJECT_NAME VARIABLE2=value2

    \b
    In .env-example file:
    VARIABLE1=
    VARIABLE2=

    The path for the environment file (or just its name if in current directory) can be specified
    by passing the -f, --filepath option:

    \b
    D run export-env -f [filepath]
    """

    helper = EnvironmentHelper()

    helper.run(
        path=ctx.obj['management'],
        project_name=ctx.obj['project_name'],
        filepath=filepath,
    )


# @run.command()
@click.option('-a', '--app', type=str, required=True, help='The app whose fixtures we must load.')
@click.option('-r', '--recursive', is_flag=True, help='Load all fixtures in directory.')
@click.argument('fixture', required=False)
@click.pass_context
def load_data(ctx, app, recursive, fixture):
    """
    Load data from fixtures.

    Loads all data contained in either .yaml or .json file within the fixtures directory of the specified app. For example:

    \b
        D run load-data --app blog articles
    
    This will load the articles.json or articles.yaml fixtures into the database.

    By specifying the --recursive flag, one can load all fixtures contained within the app's fixtures directory.
    """

    if recursive and fixture:
        log_error(DEFAULT_TOO_MANY_ARGS_ERROR)
        raise click.Abort
    elif not recursive and not fixture:
        log_error(DEFAULT_MISSING_ARGS_ERROR.format('--fixture or --recursive'))
        raise click.Abort
    
    path = ctx.obj['path']
    management = ctx.obj['management']
    
    print(path)
    print(management)


# @run.command()
@click.option('-a', '--app', type=str, required=False)
@click.option('--up/--down', default=True, help='Make or undo migrations.')
@click.option('-g', '--general', is_flag=True, help='Run migrations for all apps.')
@click.argument('options', nargs=-1, required=False)
@click.pass_context
def migrations(ctx, app, general, up, options):
    """
    Run database migrations.

    This combines both `makemigrations` and `migrate` commands into one. For example:

        D run migrations blog
    
    will accomplish the same as the following two commands:

    \b
        ./manage.py makemigrations blog && \\
        ./manage.py migrate blog

    Another thing this command seeks to accomplish is to bypass the need to navigate to
    the top of the directory in order to have access to the `manage.py` module. As long
    as the command is ran from within one of the following scopes, the command will work as intended:

    \b
        /project
        /project/project
        /project/project/app

    """

    if not app and not general:
        log_error(DEFAULT_MISSING_ARGS_ERROR.format('--app or --general'))
        raise click.Abort

    path = ctx.obj['management']

    MigrationHelper.run(path, app, general, up, options)


@run.command()
@click.option('-p', '--port', type=int, required=False, help='The port the server will listen on.')
@click.pass_context
def server(ctx, port):
    """
    Runs the development server.
    """

    ServerHelper.run(
        path=ctx.obj['management'],
        port=port
    )



from jinja2 import Template


model_form_template = Template(
    """from django.forms import forms
from ..models.{{ model.lower() }} import {{ model.capitalize() }}


class {{ model.capitalize() }}Form(forms.Form):
    class Meta:
        model = {{ model.capitalize() }}
        fields = "__all__"
""")

model_form_import_template = Template(
    """from .{{ model.lower() }} import {{ model.capitalize() }}Form"""
)



from jinja2 import Template

requirements_template = Template("""# Requirements for {{ project }}

# Django Framework
django>=2.2.1

# API Support
coreapi
djangorestframework>=3.9.1
djangorestframework-jsonapi>=2.6.0
django-filter>=2.0.0
django-rest-swagger
drf-dynamic-fields
drf-flex-fields
drf-nested-routers

# Authentication and Permissions
djangorestframework-httpsignature
djangorestframework-simplejwt
django-cors-headers
django-allauth
django-guardian
django-hijack
django-oauth-plus
django-oauth2-provider
django-otp
django-registration
django-role-permissions
django-two-factor-auth
oauth2
qrcode
twilio
django-letsencrypt

# Caching and Storage
boto3
cloudinary
django-compress
django-compressor
django-redis
django-storages
Pillow

# Debugging and Testing
coverage
django-debug-toolbar
django-extensions
django-livereload-server
health-check
jupyterlab
python-coveralls
sentry-sdk

# Database and Custom Type Support
django_polymorphic
django-recurrence
django-tenant-schemas
djongo
psycopg2-binary

# Environments
django-environ
python-dotenv

# Miscellaneous
django-widget-tweaks

# Payments
stripe

# Servers, Web-workers, and Sockets
channels
gunicorn
werkzeug

# Text Editors
django-ckeditor

""")

pipenv_template = Template("""
[[source]]
name = "pypi"
url = "https://pypi.org/simple"
verify_ssl = true

[dev-packages]

[packages]
# Django Framework
django = ">=2.2.1"

# API Support
coreapi = "*"
djangorestframework = ">=3.9.1"
djangorestframework-jsonapi = ">=2.6.0"
django-filter = ">=2.0.0"
django-rest-swagger = "*"
drf-dynamic-fields = "*"
drf-flex-fields = "*"
drf-nested-routers = "*"

# Authentication and Permissions
djangorestframework-httpsignature = "*"
djangorestframework-simplejwt = "*"
django-cors-headers = "*"
django-allauth = "*"
django-guardian = "*"
django-hijack = "*"
django-oauth-plus = "*"
django-oauth2-provider = "*"
django-otp = "*"
django-registration = "*"
django-role-permissions = "*"
django-two-factor-auth = "*"
oauth2 = "*"
qrcode = "*"
twilio = "*"
django-letsencrypt = "*"

# Caching and Storage
boto3 = "*"
cloudinary = "*"
django-compress = "*"
django-compressor = "*"
django-redis = "*"
django-storages = "*"
Pillow = "*"

# Debugging and Testing
coverage = "*"
django-debug-toolbar = "*"
django-extensions = "*"
django-livereload-server = "*"
health-check = "*"
jupyterlab = "*"
python-coveralls = "*"
sentry-sdk = "*"

# Database and Custom Type Support
django_polymorphic = "*"
django-recurrence = "*"
django-tenant-schemas = "*"
djongo = "*"
psycopg2-binary = "*"

# Environments
django-environ = "*"
python-dotenv = "*"

# Miscellaneous
django-widget-tweaks = "*"

# Payments
stripe = "*"

# Servers, Web-workers, and Sockets
channels = "*"
gunicorn = "*"
werkzeug = "*"

# Text Editors
django-ckeditor = "*"


[requires]
python_version = "3.6"

""")



from jinja2 import Template

model_admin_template = Template(
    """from django.contrib import admin
from ..models import {{ model.capitalize() }}


@admin.register({{ model.capitalize() }})
class {{ model.capitalize() }}Admin(admin.ModelAdmin):
    pass
""")


admin_user_auth_template = Template(
    """from django.contrib import admin
from django.contrib.auth.admin import (
    AdminPasswordChangeForm,
    UserAdmin as BaseUserAdmin,
    UserChangeForm,
    UserCreationForm
)
from ..models import User


@admin.register(User)
class UserAdmin(BaseUserAdmin):
    model = User
    list_display = ['id', 'username', 'name', 'is_staff', 'token']
    prepopulated_fields = {'username': ('first_name', 'last_name',)}
    form = UserChangeForm
    add_form = UserCreationForm
    change_password_form = AdminPasswordChangeForm
    readonly_fields = ('date_joined', 'last_login',)

    add_fieldsets = (
        (None, {
            'classes': ('wide',),
            'fields': ('first_name', 'last_name', 'username', 'email', 'password1', 'password2', 'photo'),
        }),
    )

    search_fields = ('email', 'username', 'first_name', 'last_name')
""")


model_admin_import_template = Template("""from .{{ model.lower() }} import {{ model.capitalize() }}Admin""")


model_admin_inline_template = Template(
    """from django.contrib import admin
from ...models import {{ model.capitalize() }}


class {{ model.capitalize() }}Inline(admin.StackedInline):
    model = {{ model.capitalize() }}
    extra = 1
""")

model_inline_import_template = Template("""from .{{ model.lower() }} import {{ model.capitalize() }}Inline""")



import random
import time


START_TIME = int(time.time()*1000)


def make_identifier():
	# https://stackoverflow.com/a/37605582/7899348

	t = int(time.time()*1000) - START_TIME
	u = random.SystemRandom().getrandbits(23)
	return (t << 23) | u


def reverse_identifier(identifier):
	t = identifier >> 23
	return t + START_TIME



from jinja2 import Template


test_case_template = Template(
    """from django.test import TestCase
from ..models import {{ model.capitalize() }}


class {{ model.capitalize() }}TestCase(TestCase):
    def setUp(self):
        \"""
        Create objects here...
        
        Example: 
        Album.objects.create(title='Djangle Today', is_compilation=True)
        \"""
        
        pass

    def test_{{ model.lower() }}_can_do_something(self):
        \"""
        Run assertions here...
        
        Example:
        album = Album.objects.get(id=1)
        self.assertEqual(album.title, 'Djangle Today')
        \"""
        
        pass

""")

test_case_import_template = Template("""from .{{ model.lower() }} import {{ model.capitalize() }}TestCase""")



import click
import fileinput
import inflection
import os
from django_clite.cli import log_success, sanitized_string, log_info
from django_clite.cli.commands.base_helper import BaseHelper
from django_clite.cli.templates.model import (
    model_field_template,
    model_import_template,
    model_template,
    sql_view_template
)

class SQLHelper(BaseHelper):
  pass



from django_clite.cli import log_success
from django_clite.cli.commands.base_helper import BaseHelper
from django_clite.cli.templates.admin import (
    admin_user_auth_template,
    model_admin_import_template,
    model_admin_inline_template,
    model_admin_template,
    model_inline_import_template,
)


class AdminHelper(BaseHelper):

    def create(self, **kwargs):
        model = self.check_noun(kwargs['model'])

        path = kwargs['path']

        filename = f"{model.lower()}.py"

        template = model_admin_template

        template_import = model_admin_import_template

        message = "Successfully created admin model"

        if kwargs['inline']:
            template = model_admin_inline_template
            template_import = model_inline_import_template
            message = "Successfully created admin inline"

        self.parse_and_create(
            filename=filename,
            model=model,
            path=path,
            template=template,
            dry=kwargs['dry']
        )

        self.add_import(**kwargs, template=template_import)

        log_success(message)

    def delete(self, **kwargs):
        model = self.check_noun(kwargs['model'])

        filename = f"{model.lower()}.py"

        template = model_admin_import_template

        message = "Successfully deleted admin model"

        if kwargs['inline']:
            template = model_inline_import_template
            message = "Successfully deleted admin inline"

        if self.destroy(filename=filename, **kwargs):
            self.remove_import(template=template, **kwargs)
            log_success(message)

    @classmethod
    def create_auth_user(cls, **kwargs):
        kwargs['model'] = 'User'

        kwargs['filename'] = 'user.py'

        kwargs['path'] = f"{kwargs['path']}/admin/"

        cls.parse_and_create(
            model=kwargs['model'],
            filename=kwargs['filename'],
            project_name=kwargs['project'],
            template=admin_user_auth_template,
            path=kwargs['path']
        )

        cls.add_import(**kwargs, template=model_admin_import_template)
# end class



from jinja2 import Template

manager_template = Template("""from django.db import models, transaction


class Custom{{ model.capitalize() }}QuerySet(models.QuerySet):
    # Available on both Manager and QuerySet.
    def public_method(self):
        raise NotImplementedError

    # Available only on QuerySet.
    def _private_method(self):
        raise NotImplementedError


class {{ model.capitalize() }}Manager(models.Manager):

    \"""
    Applying a custom QuerySet.
    def get_queryset(self):
        return Custom{{ model.capitalize() }}QuerySet(self.model, using=self._db)


    In your desired model, assign the manager to one of your model attributes.
    objects = {{ model.capitalize() }}Manager()
    \"""

    def get_queryset(self):
        return super().get_queryset()

    @transaction.atomic
    def create_{{ model.lower() }}(self, **kwargs):
      raise NotImplementedError
""")


manager_import_template = Template(
    """from .{{ model.lower() }} import {{ model.capitalize() }}Manager"""
)



from jinja2 import Template

bitbucket_pipeline_template = Template(
  """
image: python:3.7.2
pipelines: 
  default: 
    - step:
        script:
          - pip install django
          - python manage.py test
      services: 
        - postgres 
        - redis
definitions: 
  services: 
    postgres: 
      image: postgres
    redis:
      image: redis
""")



from django_clite.cli import log_success
from django_clite.cli.commands.base_helper import BaseHelper
from django_clite.cli.templates.manager import (
    manager_template,
    manager_import_template
)


class ManagerHelper(BaseHelper):

    def create(self, **kwargs):
        model = self.check_noun(kwargs['model'])

        path = kwargs['path']

        filename = f"{model.lower()}.py"

        template = manager_template

        template_import = manager_import_template

        self.parse_and_create(
            filename=filename,
            model=model,
            path=path,
            template=template,
            dry=kwargs['dry']
        )

        self.add_import(**kwargs, template=template_import)

        log_success("Successfully created manager class.")

    def delete(self, **kwargs):
        model = self.check_noun(kwargs['model'])

        filename = f"{model.lower()}.py"

        template = manager_import_template

        if self.destroy(filename=filename, **kwargs):

            self.remove_import(template=template, **kwargs)

            log_success('Successfully deleted manager.')

# end class



from jinja2 import Template

settings_template = Template("""
# Django settings for {{ project_name }} project.


import os

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.environ.get('SECRET_KEY')

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = os.environ.get('DEBUG') == 'True' and os.environ.get('DATABASE_URL') is None

ALLOWED_HOSTS = [] if os.environ.get('LOCAL_MACHINE') else ['{{ domain }}']

INTERNAL_IPS = '127.0.0.1'


# Application definition

SITE_ID = 1

INSTALLED_APPS = [
    'django.contrib.admin',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.sites',
    'django_extensions',

    # Authentication and Authorisation
    'allauth',
    'allauth.account',
    'allauth.socialaccount',
    'django_otp',
    'django_otp.plugins.otp_static',
    'django_otp.plugins.otp_totp',
    'rolepermissions',
    'guardian',

    # Django REST Framework
    'rest_framework',
    'rest_framework.authtoken',
    'rest_framework_httpsignature',
    'rest_framework_swagger',
    
    # File storage and caching
    'storages',
    'compressor',
    'django_redis',

    # Extras
    'corsheaders',
    'debug_toolbar',

    {% for app in apps %}
    '{{ project_name }}.{{ app }}',
    {% endfor %}
]

MIDDLEWARE = [
    'django.contrib.sites.middleware.CurrentSiteMiddleware',
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django_otp.middleware.OTPMiddleware',
    'debug_toolbar.middleware.DebugToolbarMiddleware',
    'corsheaders.middleware.CorsMiddleware',
    
    {% for middleware in middlewares %}
    '{{ middleware }}',
    {% endfor %}
]

ROOT_URLCONF = '{{ project_name }}.urls'

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [
            {% for app in apps %}
            os.path.join(BASE_DIR, '{{ project_name }}/{{ app }}/templates'),
            {% endfor %}
        ],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
        },
    },
]

WSGI_APPLICATION = '{{ project_name }}.wsgi.application'


# Database

if "IN_DOCKER" in os.environ:
    # Stuff for when running in Docker-compose.

    CELERY_BROKER_URL = 'redis://redis:6379/1'
    CELERY_RESULT_BACKEND = 'redis://redis:6379/1'

    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql_psycopg2',
            'NAME': os.environ.get('DOCKER_DB_NAME'),
            'USER': os.environ.get('DOCKER_DB_USER'),
            'PASSWORD': os.environ.get('DOCKER_DB_PASSWORD'),
            'HOST': "db",
            'PORT': 5432,
        }
    }

elif "DATABASE_URL" in os.environ:
    USER, PASSWORD, HOST, PORT, NAME = re.match("^postgres://(?P<username>.*?)\:(?P<password>.*?)\@(?P<host>.*?)\:(?P<port>\d+)\/(?P<db>.*?)$", os.environ.get("DATABASE_URL", "")).groups()

    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql_psycopg2',
            'NAME': NAME,
            'USER': USER,
            'PASSWORD': PASSWORD,
            'HOST': HOST,
            'PORT': int(PORT),
        }
    }

elif "LOCAL_MACHINE" in os.environ:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.postgresql',
            'NAME': os.environ.get('DB_NAME'),
            'USER': os.environ.get('DB_USER'),
            'HOST': os.environ.get('DB_HOST'),
            'PORT': os.environ.get('DB_PORT'),
            'PASSWORD': os.environ.get('DB_PASSWORD'),
        }
     }

else:
    DATABASES = {
        'default': {
            'ENGINE': 'django.db.backends.sqlite3',
            'NAME': 'db.sqlite3'
        }
    }

if 'EMAIL_HOST_PASSWORD' in os.environ:
    EMAIL_BACKEND = "django.core.mail.backends.smtp.EmailBackend"
    EMAIL_USE_TLS = True
    EMAIL_HOST = os.getenv('EMAIL_HOST', None)
    EMAIL_HOST_USER = os.getenv('EMAIL_HOST_USER', None)
    EMAIL_HOST_PASSWORD = os.getenv('EMAIL_HOST_PASSWORD', None)
    EMAIL_PORT = 587
    
    SENDGRID_API_KEY = os.getenv('SENDGRID_API_KEY')
else:
    EMAIL_BACKEND = "django.core.mail.backends.console.EmailBackend"

SERVER_EMAIL = os.environ.get('SERVER_EMAIL')

EMAIL_RELAY_DOMAIN = os.environ.get('EMAIL_RELAY_DOMAIN')


# Password validation

{% if custom_auth %}
AUTH_USER_MODEL = 'authentication.User'

ACCOUNT_USERNAME_VALIDATORS = '{{ project_name }}.authentication.models.validators.username'
{% endif %}

AUTHENTICATION_BACKENDS = [
    'django.contrib.auth.backends.ModelBackend',
    'guardian.backends.ObjectPermissionBackend',
    'allauth.account.auth_backends.AuthenticationBackend',
]

AUTH_PASSWORD_VALIDATORS = [
    {
        'NAME': 'django.contrib.auth.password_validation.UserAttributeSimilarityValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.MinimumLengthValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.CommonPasswordValidator',
    },
    {
        'NAME': 'django.contrib.auth.password_validation.NumericPasswordValidator',
    },
]

# https://django-allauth.readthedocs.io/en/latest/providers.html
SOCIALACCOUNT_PROVIDERS = {}

LOGIN_REDIRECT_URL = '/accounts/me'

ROLEPERMISSIONS_MODULE = '{{ project_name }}.roles'

GUARDIAN_RAISE_403 = True

OTP_TOTP_ISSUER = os.environ.get('OTP_ISSUER')


# Internationalization

LANGUAGE_CODE = 'en-us'

TIME_ZONE = 'UTC'

USE_I18N = True

USE_L10N = True

USE_TZ = True


# Static files (CSS, JavaScript, Images)

AWS_ACCESS_KEY_ID = os.environ.get('AWS_ACCESS_KEY_ID')

AWS_SECRET_ACCESS_KEY = os.environ.get('AWS_SECRET_ACCESS_KEY')

AWS_STORAGE_BUCKET_NAME = os.environ.get('AWS_BUCKET_NAME')

AWS_S3_ENDPOINT_URL = os.environ.get('AWS_ENDPOINT_URL')

AWS_S3_CUSTOM_DOMAIN = os.environ.get('AWS_CUSTOM_DOMAIN')

AWS_LOCATION = os.environ.get('AWS_LOCATION')

AWS_STATIC_LOCATION = os.environ.get('AWS_STATIC_LOCATION')

AWS_PUBLIC_MEDIA_LOCATION = os.environ.get('AWS_MEDIA_LOCATION')

AWS_PRIVATE_MEDIA_LOCATION = os.environ.get('AWS_PRIVATE_MEDIA_LOCATION')

AWS_DEFAULT_ACL = 'public-read'

AWS_S3_OBJECT_PARAMETERS = {
    'CacheControl': 'max-age=86400',
}

AWS_IS_GZIPPED = True

DEFAULT_FILE_STORAGE = 'storages.backends.s3boto3.S3Boto3Storage'

STATICFILES_STORAGE = 'storages.backends.s3boto3.S3Boto3Storage'

STATICFILES_FINDERS = [
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
    'compressor.finders.CompressorFinder'
]

STATIC_URL = f'{AWS_S3_ENDPOINT_URL}/'

STATIC_ROOT = 'staticfiles/'

MEDIA_ROOT = 'mediafiles/'

MEDIA_URL = '/files/'

COMPRESS_ROOT = ''


# Django REST Framework, JWT, and Swagger
# This implementation is JSON-API compatible

REST_FRAMEWORK = {
    'PAGE_SIZE': 25,

    'DEFAULT_PAGINATION_CLASS': 'rest_framework_json_api.pagination.JsonApiPageNumberPagination',

    'EXCEPTION_HANDLER': 'rest_framework_json_api.exceptions.exception_handler',

    'DEFAULT_AUTHENTICATION_CLASSES': (
        'rest_framework.authentication.SessionAuthentication',
        'rest_framework.authentication.TokenAuthentication',
        'rest_framework_simplejwt.authentication.JWTAuthentication',
    ),

    'DEFAULT_PERMISSION_CLASSES': [
        'rest_framework.permissions.IsAuthenticated',
    ],

    'DEFAULT_PARSER_CLASSES': (
        'rest_framework_json_api.parsers.JSONParser',
        'rest_framework.parsers.FormParser',
        'rest_framework.parsers.MultiPartParser'
    ),

    'DEFAULT_RENDERER_CLASSES': (
        'rest_framework_json_api.renderers.JSONRenderer',
    ),

    'DEFAULT_METADATA_CLASS': 'rest_framework_json_api.metadata.JSONAPIMetadata',

    'DEFAULT_FILTER_BACKENDS': (
        'rest_framework_json_api.filters.QueryParameterValidationFilter',
        'rest_framework_json_api.filters.OrderingFilter',
        'rest_framework_json_api.django_filters.DjangoFilterBackend',
        'rest_framework.filters.SearchFilter',
    ),

    'SEARCH_PARAM': 'filter[search]',

    'TEST_REQUEST_RENDERER_CLASSES': (
        'rest_framework_json_api.renderers.JSONRenderer',
    ),

    'TEST_REQUEST_DEFAULT_FORMAT': 'vnd.api+json'
}

JSON_API_FORMAT_FIELD_NAMES = 'dasherize'

JSON_API_FORMAT_TYPES = 'dasherize'

JSON_API_PLURALIZE_TYPES = False

SIMPLE_JWT = {
    'ACCESS_TOKEN_LIFETIME': timedelta(minutes=5),
    'REFRESH_TOKEN_LIFETIME': timedelta(days=1),
    'ROTATE_REFRESH_TOKENS': False,
    'BLACKLIST_AFTER_ROTATION': True,

    'ALGORITHM': 'HS256',
    'SIGNING_KEY': SECRET_KEY,
    'VERIFYING_KEY': None,

    'AUTH_HEADER_TYPES': ('Bearer',),
    'USER_ID_FIELD': 'id',
    'USER_ID_CLAIM': 'user_id',

    'AUTH_TOKEN_CLASSES': ('rest_framework_simplejwt.tokens.AccessToken',),
    'TOKEN_TYPE_CLAIM': 'token_type',

    'SLIDING_TOKEN_REFRESH_EXP_CLAIM': 'refresh_exp',
    'SLIDING_TOKEN_LIFETIME': timedelta(minutes=5),
    'SLIDING_TOKEN_REFRESH_LIFETIME': timedelta(days=1),
}

SWAGGER_SETTINGS = {
    "LOGIN_URL": 'rest_framework:login',
    "LOGOUT_URL": 'rest_framework:logout',
    "APIS_SORTER": "alpha",
    "DOC_EXPANSION": "None",
    "USE_SESSION_AUTH": True,

    "SHOW_REQUEST_HEADERS": True,
    "CUSTOM_HEADERS": {
      "Accept": "application/vnd.api+json",
      "Content-type": "application/vnd.api+json"
    },

    "ENCODING": "application/vnd.api+json",

    "JSON_EDITOR": True,
    "OPERATIONS_SORTER": 'method',
    "SECURITY_DEFINITIONS": {
        "api_key": {
            "type": "apiKey",
            "name": "Token ",
            "in": "header"
        }
    },
}


# Error logging and reporting with Sentry

if 'DATABASE_URL' in os.environ:
    # Advanced error reporting in production.
    # Include settings when DOKKU environment is detected.

    import sentry_sdk
    from sentry_sdk.integrations.django import DjangoIntegration

    sentry_sdk.init(
        dsn=os.environ.get('SENTRY_DSN'),
        integrations=[DjangoIntegration()]
    )

""")



from django_clite.cli import log_success
from django_clite.cli.commands.base_helper import BaseHelper
from django_clite.cli.templates.view import (
    default_function_view_template,
    default_function_view_import_template,
    default_class_view_template,
    default_class_view_import_template
)


class ViewHelper(BaseHelper):

    def create(self, **kwargs):
        model = self.check_noun(kwargs['model'])

        kwargs['model'] = model

        kwargs['name'] = model

        template = default_class_view_template

        template_import = default_class_view_import_template

        filename = f"{model.lower()}.py"

        if not kwargs['detail'] and not kwargs['list']:
            template = default_function_view_template
            template_import = default_function_view_import_template

        if kwargs['list']:
            filename = f"{model.lower()}_list.py"
        elif kwargs['detail']:
            filename = f"{model.lower()}_detail.py"

        self.parse_and_create(
            filename=filename,
            template=template,
            **kwargs
        )

        self.add_import(
            **kwargs,
            template=template_import,
        )

        log_success(f"Successfully created view.")

    def delete(self, **kwargs):
        model = self.check_noun(kwargs['model'])

        kwargs['model'] = model

        kwargs['name'] = model

        filename = f"{model.lower()}.py"

        if kwargs['list']:
            filename = f"{model.lower()}_list.py"
        elif kwargs['detail']:
            filename = f"{model.lower()}_detail.py"

        if self.destroy(filename=filename, **kwargs):

            if kwargs['list'] or kwargs['detail']:
                self.remove_import(
                    template=default_class_view_import_template,
                    model=model,
                    list=kwargs['list'],
                    detail=kwargs['detail'],
                    path=kwargs['path']
                )
            else:
                self.remove_import(template=default_function_view_import_template, **kwargs)

            log_success(f'Successfully deleted view.')

# end class



import inflection
from django_clite.cli import log_success
from django_clite.cli.commands.base_helper import BaseHelper
from django_clite.cli.templates.viewset import (
    viewset_template,
    viewset_import_template
)


class ViewSetHelper(BaseHelper):

    def create(self, **kwargs):
        model = self.check_noun(kwargs['model'])

        path = kwargs['path']

        filename = f"{model.lower()}.py"

        # TODO: Ensure serializer already exists

        self.parse_and_create(
            filename=filename,
            model=model,
            template=viewset_template,
            read_only=kwargs['read_only'],
            route=inflection.pluralize(model),
            path=path,
            dry=kwargs['dry']
        )

        self.add_import(
            model=model,
            template=viewset_import_template,
            path=path,
            dry=kwargs['dry']
        )

        log_success("Successfully created viewset.")

    def delete(self, **kwargs):
        model = self.check_noun(kwargs['model'])

        filename = f"{model.lower()}.py"

        template = viewset_import_template

        if self.destroy(filename=filename, **kwargs):

            self.remove_import(template=template, **kwargs)

            log_success('Successfully deleted viewset.')

    @classmethod
    def create_auth_user(cls, **kwargs):
        kwargs['model'] = 'User'

        kwargs['filename'] = 'user.py'

        kwargs['path'] = f"{kwargs['path']}/viewsets/"

        kwargs['route'] = inflection.pluralize(f"{kwargs['model']}")

        cls.parse_and_create(
            model=kwargs['model'],
            route=kwargs['route'],
            filename=kwargs['filename'],
            project_name=kwargs['project'],
            template=viewset_template,
            path=kwargs['path']
        )

        cls.add_import(**kwargs, template=viewset_import_template)

# end class



from jinja2 import Template


serializer_template = Template(
    """from rest_framework import serializers
{% if model %}from ..models import {{ model.capitalize() }}
{% endif %}

class {{ model.capitalize() }}Serializer(serializers.ModelSerializer):
    
    # Add related fields below:
    # Example relation fields are:
    # -- HyperlinkedIdentityField
    # -- HyperlinkedRelatedField
    # -- PrimaryKeyRelatedField
    # -- SlugRelatedField
    # -- StringRelatedField
    
    # You can also create a custom serializer, like so:
    # likes = LikeSerializer(many=True)

    class Meta:
        model = {{ model.capitalize() }}
        fields = "__all__"
""")

serializer_auth_user_template = Template(
    """from rest_framework import serializers
from ..models import User


class UserSerializer(serializers.ModelSerializer):

    class Meta:
        model = User
        exclude = ('date_joined', 'updated_at', 'last_login',
                   'is_superuser', 'is_staff', 'user_permissions', 'password')
""")

serializer_import_template = Template(
    """from .{{ model.lower() }} import {{ model.capitalize() }}Serializer"""
)



import os
import subprocess


DEFAULT_LOAD_FIXTURES_COMMAND = ['python3', 'manage.py', 'loaddata']


class FixtureHelper(object):
  
  @classmethod
  def load_one(cls, path, fixture):
    os.chdir(path)

  @classmethod
  def load_all(cls, path):
    os.chdir(path)



"""
django-clite

Description:
A CLI that handles creating and managing Django projects
"""

import click
from django_clite import AliasedGroup
from django_clite.cli.commands import create
from django_clite.cli.commands import destroy
from django_clite.cli.commands import generate
from django_clite.cli.commands import run


CONTEXT_SETTINGS = dict(token_normalize_func=lambda x: x.lower())


@click.command(cls=AliasedGroup)
@click.pass_context
def main(ctx):
    """
    django-clite

    A CLI to handle the creation and management of your Django projects.
    The CLI has some options about how your project should be structured in order for it to maximize the
    amount of automatic configuration it can provide you. Since Django itself adheres to configuration over
    convention, you are free to bypass conventions of the CLI if you so choose.
    """


"""
Commands should be added as sub-commands of the main click group.
This ensures sub-commands can be chained and
run as `django-clite SUB-COMMAND COMMAND`
"""
main.add_command(create)
main.add_command(destroy)
main.add_command(generate)
main.add_command(run)

if __name__ == '__main__':
    main()



from jinja2 import Template

model_field_template = Template("""{{ name }} = {% if not special %}models.{% endif %}{{ type }}({{ options }})""")


model_admin_template = Template("""admin.register({{ model.capitalize() }}
class {{ model.capitalize() }}Admin(admin.ModelAdmin):
    pass
""")


model_form_template = Template("""from django.forms import forms
from {{ app }}.models.{{ model.lower() }} import {{ model.capitalize() }}


class {{ model.capitalize() }}Form(forms.Form):
    class Meta:
        model = {{ model.capitalize() }}
        fields = "__all__"
""")


model_template = Template("""import uuid
from django.db import models
{% for model in imports %}{% if model %}from .{{ model.lower() }} import {{ model.capitalize() }}{% endif %}
{% endfor %}
{% if base %}from {{ base[0] }} import {{ base[1] }}\n\n{% endif %}
class {{ model.capitalize() }}({% if base %}{{ base[1] }}{% else %}models.Model{% endif %}):
    {% for field in fields %}{{ field }}
    {% endfor %}
    # Default fields. Used for record-keeping.
    uuid = models.UUIDField(default=uuid.uuid4, editable=False)
    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    updated_at = models.DateTimeField(auto_now=True, editable=False)

    class Meta:
        db_table = '{{ db_table.lower() }}'
        ordering = ['-created_at']
        {% if abstract %}abstract = True\n{% endif %}
    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)

    def __str__(self):
        return f'{self.uuid}'
""")

sql_view_template = Template("""import uuid
from django.db import models
{% for model in imports %}{% if model %}from .{{ model.lower() }} import {{ model.capitalize() }}{% endif %}
{% endfor %}
{% if base %}from {{ base[0] }} import {{ base[1] }}\n\n{% endif %}
class {{ model.capitalize() }}({% if base %}{{ base[1] }}{% else %}models.Model{% endif %}):
    {% for field in fields %}{{ field }}
    {% endfor %}
    class Meta:
        db_table = '{{ db_table.lower() }}'
        managed = False
""")

sql_view_management_template = ("""
migrations.RunSQL(
    \"""
    DROP VIEW IF EXISTS {{ db_table.lower() }};
    CREATE OR REPLACE VIEW {{ db_table.lower() }} AS
        SELECT 
            {% for field in fields %}
                {{ field }}
            {% endfor %} 
        FROM {{ source }};
    \"""
)
""")

auth_user_model_template = Template("""import uuid
from django.db import models
from django.contrib.auth.models import AbstractUser
from rest_framework.authtoken.models import Token


class User(AbstractUser):
    photo = models.ImageField(upload_to='users/profiles/', blank=True)

    # Default fields. Used for record-keeping.
    uuid = models.UUIDField(default=uuid.uuid4, editable=False)
    created_at = models.DateTimeField(auto_now_add=True, editable=False)
    updated_at = models.DateTimeField(auto_now=True, editable=False)

    class Meta:
        db_table = 'authentication_users'
        ordering = ['-created_at']

    @property
    def name(self):
        return f"{self.first_name} {self.last_name}"
    
    @property
    def token(self):
        return Token.objects.get_or_create(user_id=self.id)

    def clean(self):
        \"""
        Ensure username is always lowercase.
        \"""
        super().clean()
        self.username = self.username.lower()

    def save(self, *args, **kwargs):
        new_user = False
        if not self.id:
            new_user = True
        super().save(*args, **kwargs)
        Token.objects.get_or_create(user_id=self.id)
        
        \"""
        Notifying user that account was successfully created
        if new_user:
            self.email_user(subject='Message from {{ project_name }}',
                            message='Your account was successfully created.')
        \"""

    def __str__(self):
        return self.username
""")

model_import_template = Template("""from .{{ model.lower() }} import {{ model.capitalize() }}""")



# django-clite
from .admin_helper import AdminHelper
from .form_helper import FormHelper
from .manager_helper import ManagerHelper
from .model_helper import ModelHelper
from .serializer_helper import SerializerHelper
from .template_helper import TemplateHelper
from .view_helper import ViewHelper
from .viewset_helper import ViewSetHelper
from .test_helper import TestHelper
from .sql_helper import SQLHelper



import os
import subprocess
import sys
from django_clite.cli import log_error, log_info, log_success, sanitized_string
from django_clite.cli.commands.base_helper import BaseHelper
from django_clite.cli.commands.generator.helpers import (
    AdminHelper,
    ModelHelper,
    TestHelper,
    SerializerHelper,
    ViewSetHelper,
)
from django_clite.cli.templates.git_repo.readme import readme_template
from django_clite.cli.templates.git_repo.gitignore import git_ignore_template
from django_clite.cli.templates.router import router_template
from django_clite.cli.templates.urls import app_urls_template
from django_clite.cli.templates.env import env_template
from django_clite.cli.templates.requirements import (
    requirements_template,
    pipenv_template
)
from django_clite.cli.templates.docker import (
    docker_compose_template,
    dockerfile_template
)
from django_clite.cli.templates.dokku import (
    app_template,
    dokku_checks_template,
    dokku_scale_template,
    procfile_template
)


DEFAULT_ERRORS = {
    "project": "Unable to create project. Will exit.",
    "app": "Unable to create app. Skipping...",
    "clean": "Unable to remove default files. Skipping...",
    "repo": "Unable to initialize repository. Skipping...",
    "touch": "Unable to create default files. Skipping...",
}

DEFAULT_APP_PACKAGES = {
    'admin', 'fixtures', 'forms', 'middleware', 'models', 'serializers',
    'templates', 'tests', 'views', 'viewsets'
}

DEFAULT_CWD = '.'

DEFAULT_PREVIOUS_WD = '..'


class CreatorHelper(object):

    helper = BaseHelper()

    @classmethod
    def create_app(cls, **kwargs):
        """
        Creates an app and its respective packages
        """
        project_name = sanitized_string(kwargs['project'])
        app_name = sanitized_string(kwargs['app_name'])

        # Create application structure
        try:
            subprocess.check_output(['django-admin', 'startapp', app_name])
        except subprocess.CalledProcessError:
            log_error(DEFAULT_ERRORS['app'])
            return

        # Enter app directory
        os.chdir(app_name)

        # Remove unused modules
        try:
            subprocess.check_output(['rm', 'admin.py', 'models.py', 'tests.py', 'views.py', '__init__.py'])
        except subprocess.CalledProcessError:
            log_error(DEFAULT_ERRORS['clean'])

        # Add urls.py module
        try:
            cls.helper.parse_and_create(
                template=app_urls_template,
                project=project_name,
                app_name=app_name,
                filename='urls.py',
                path=DEFAULT_CWD
            )
            cls.helper.create_file(
                path=DEFAULT_CWD,
                filename='__init__.py',
                file_content=f'# application:{app_name}'
            )
        except subprocess.CalledProcessError:
            log_error(DEFAULT_ERRORS['touch'])

        # Add packages to app structure
        for package in DEFAULT_APP_PACKAGES:
            cls.create_app_package(package_name=package, app_name=app_name)
            os.chdir(DEFAULT_PREVIOUS_WD)

        try:
            if kwargs['custom_auth']:
                cls.handle_custom_auth(**kwargs)
        except KeyError:
            pass

        os.chdir(DEFAULT_PREVIOUS_WD)
        log_success(f'Successfully created application: {app_name}')

    @classmethod
    def create_app_package(cls, **kwargs):
        """
        Create an app package and any associated sub-packages or modules
        """
        app_name = sanitized_string(kwargs['app_name'])
        package_name = sanitized_string(kwargs['package_name'])
        cls.create_package(package_name=package_name, app_name=app_name)

        # Do customizations
        os.chdir(package_name)
        cls.create_package(package_name='helpers', app_name=app_name)
        if package_name == 'admin':
            cls.create_package(package_name='actions', app_name=app_name)
            cls.create_package(package_name='inlines', app_name=app_name)
            cls.create_package(package_name='permissions', app_name=app_name)
        if package_name == 'models':
            cls.create_package(package_name='managers', app_name=app_name)
            cls.create_package(package_name='validators', app_name=app_name)
            cls.create_package(package_name='signals', app_name=app_name)
        if package_name == 'viewsets':
            cls.helper.parse_and_create(
                template=router_template,
                app_name=app_name,
                filename='router.py',
                path=DEFAULT_CWD
            )

    @classmethod
    def create_package(cls, **kwargs):
        """
        Create a package and any associated modules
        """
        app_name = sanitized_string(kwargs['app_name'])
        package_name = sanitized_string(kwargs['package_name'])
        content = f'# {app_name}:{package_name}'

        os.mkdir(package_name)
        os.chdir(package_name)
        cls.helper.create_file(path=DEFAULT_CWD, filename='__init__.py', file_content=content)
        os.chdir(DEFAULT_PREVIOUS_WD)

    @classmethod
    def create_project(cls, **kwargs):
        """
        Creates a project structure and its dependent packages
        """

        project_name = sanitized_string(kwargs['project'])

        # Creating project
        try:
            log_info(f"Attempting to create project: {project_name}...")
            subprocess.check_output(['django-admin', 'startproject', project_name])
        except subprocess.CalledProcessError:
            log_error(DEFAULT_ERRORS['project'])
            sys.exit(-1)

        # Add customizations
        os.chdir(project_name)

        cls.handle_docs_and_misc(**kwargs)

        cls.handle_dependencies(**kwargs)

        if kwargs['dokku']:
            cls.handle_dokku(**kwargs)

        if kwargs['docker']:
            cls.handle_docker(**kwargs)

        # Inside project/project/
        os.chdir(project_name)

        if kwargs['apps']:
            for app in kwargs['apps']:
                cls.create_app(app_name=app, **kwargs)

        # cd ../../
        os.chdir(DEFAULT_PREVIOUS_WD)

        try:
            cls.handle_git()
        except subprocess.CalledProcessError:
            log_error(DEFAULT_ERRORS['repo'])

        log_success(f"Created project: {project_name}")

    @classmethod
    def handle_custom_auth(cls, **kwargs):
        # Terra Nullius...

        kwargs['project'] = sanitized_string(kwargs['project'])

        kwargs['path'] = os.getcwd()

        ModelHelper().create_auth_user(**kwargs)

        AdminHelper().create_auth_user(**kwargs)

        SerializerHelper().create_auth_user(**kwargs)

        ViewSetHelper().create_auth_user(**kwargs)

        TestHelper().create_auth_user(**kwargs)

        # TODO: Add configurations to settings.py
        log_success('Created authentication app for custom auth')

    @classmethod
    def handle_dokku(cls, **kwargs):
        try:
            project_name = sanitized_string(kwargs['project'])

            os.mkdir('dokku')
            os.chdir('dokku')

            # app.json
            cls.helper.parse_and_create(
                template=app_template,
                project=project_name,
                filename='app.json',
                path=DEFAULT_CWD
            )

            # Procfile
            cls.helper.parse_and_create(
                template=procfile_template,
                project=project_name,
                filename='Procfile',
                path=DEFAULT_CWD
            )

            # CHECKS
            cls.helper.parse_and_create(
                template=dokku_checks_template,
                project=project_name,
                filename='CHECKS',
                wait=20,
                timeout=60,
                path=DEFAULT_CWD
            )

            cls.helper.parse_and_create(
                template=dokku_checks_template,
                project=project_name,
                filename='CHECKS',
                wait=20,
                timeout=60,
                path=DEFAULT_PREVIOUS_WD
            )

            # DOKKU_SCALE
            cls.helper.parse_and_create(
                template=dokku_scale_template,
                project=project_name,
                filename='DOKKU_SCALE',
                web=1,
                path=DEFAULT_CWD
            )

            os.chdir(DEFAULT_PREVIOUS_WD)
        except subprocess.CalledProcessError:
            log_error(DEFAULT_ERRORS['touch'])

    @classmethod
    def handle_docker(cls, **kwargs):
        project_name = sanitized_string(kwargs['project'])

        # docker-compose
        cls.helper.parse_and_create(
            template=docker_compose_template,
            project=project_name,
            filename='docker-compose.yml',
            path=DEFAULT_CWD
        )

        # Dockerfile
        cls.helper.parse_and_create(
            template=dockerfile_template,
            project=project_name,
            filename='Dockerfile',
            path=project_name
        )

    @classmethod
    def handle_dependencies(cls, **kwargs):
        project_name = sanitized_string(kwargs['project'])

        # Pipfile
        cls.helper.parse_and_create(
            template=pipenv_template,
            project=project_name,
            filename='Pipfile',
            path=DEFAULT_CWD,
        )

        # Requirements
        cls.helper.parse_and_create(
            template=requirements_template,
            project=project_name,
            filename='requirements.txt',
            path=DEFAULT_CWD
        )

    @classmethod
    def handle_docs_and_misc(cls, **kwargs):
        project_name = sanitized_string(kwargs['project'])

        # README
        cls.helper.parse_and_create(
            template=readme_template,
            project=project_name,
            filename='README.md',
            path=DEFAULT_CWD
        )

        # Git Ignore
        cls.helper.parse_and_create(
            template=git_ignore_template,
            project=project_name,
            filename='.gitignore',
            path=DEFAULT_CWD
        )

        # Environment files
        cls.helper.parse_and_create(
            path=DEFAULT_CWD,
            filename='.env',
            template=env_template
        )

        cls.helper.parse_and_create(
            path=DEFAULT_CWD,
            filename='.env-example',
            template=env_template
        )

    @classmethod
    def handle_git(cls):
        subprocess.check_output(['git', 'init'])
        subprocess.check_output(['git', 'add', '--all'])
        subprocess.check_output(['git', 'commit', '-m', 'Initial commit'])
        log_success('Successfully initialized git repository')
# end class



import click
import fileinput
import inflection
import os
from django_clite.cli import log_error, log_info


DEFAULT_IMPORT_WARNING = "Module already imported. Skipping..."

DEFAULT_OVERRIDE_WARNING = "Module already exists. Override file?"

DEFAULT_NOUN_NUMBER_WARNING = "Tip: resource name should be singular."

DEFAULT_NOUN_NUMBER_OPTION = "Change resource name from {} to {}?"

DEFAULT_PARSED_CONTENT_LOG = """Filename: {}\nFilepath: {}\n\n---- Begin content ----\n{}\n---- End content ----"""

DEFAULT_DESTROY_LOG = """Will delete...\nFilename: {}\nFilepath: {}\n\nWill also remove imports in __init__.py"""


class BaseHelper(object):

    @classmethod
    def add_import(cls, **kwargs):
        content = None

        try:
            is_dry = kwargs['dry']
        except KeyError:
            is_dry = False

        if not is_dry:
            filename = '__init__.py'
            path = kwargs['path']
            template = kwargs['template']

            try:
                content = template.render(**kwargs)
            except Exception:
                raise EnvironmentError

            try:
                os.chdir(path)

                # Prevent duplicate imports
                for line in fileinput.input(filename):
                    if content in line:
                        log_error(DEFAULT_IMPORT_WARNING)
                        fileinput.close()
                        return
                fileinput.close()

                with open(filename, 'a') as file:
                    file.write(content)
                    file.write('\n')
                file.close()
            except FileNotFoundError:
                raise FileNotFoundError

        return content

    @classmethod
    def check_noun(cls, noun):
        """
        Checks whether a noun is plural or singular and gives
        the option to change the noun from plural to singular.
        """
        noun = noun.lower()

        if inflection.pluralize(noun) == noun:
            log_info(DEFAULT_NOUN_NUMBER_WARNING)
            singular = inflection.singularize(noun)

            if click.confirm(DEFAULT_NOUN_NUMBER_OPTION.format(noun, singular)):
                noun = singular
        return noun

    @classmethod
    def create(cls, **kwargs):
        raise NotImplementedError

    @classmethod
    def create_file(cls, path, filename, file_content):
        directory = os.getcwd()

        try:
            os.chdir(path)
        except FileNotFoundError:
            os.makedirs(path)
            os.chdir(path)

        try:
            with open(filename, 'x') as file:
                file.write(file_content)
                file.write('\n')
            file.close()
            os.chdir(directory)
        except FileExistsError:
            if click.confirm(DEFAULT_OVERRIDE_WARNING):
                with open(filename, 'w') as file:
                    file.write(file_content)
                    file.write('\n')
                file.close()
                os.chdir(directory)
                return
            raise click.Abort

    @classmethod
    def destroy(cls, path, filename, **kwargs):
        try:
            is_dry = kwargs['dry']
        except KeyError:
            is_dry = False

        if is_dry:
            kwargs['filename'] = filename
            click.echo(DEFAULT_DESTROY_LOG.format(filename, path))
            return False
        else:
            try:
                os.chdir(path)
                os.remove(filename)
            except FileNotFoundError:
                log_error("File does not exist.")
                return False
        return True

    @classmethod
    def log_dry(cls, **kwargs):
        """
        Responsible for logging commands to the console.
        Used when --dry flag is specified.
        """
        click.echo(DEFAULT_PARSED_CONTENT_LOG.format(
            kwargs['filename'],
            kwargs['path'],
            kwargs['content']
        ))

    @classmethod
    def noun_is_plural(cls, text):
        return inflection.pluralize(text) == text

    @classmethod
    def parse_template(cls, template, **kwargs):
        return template.render(**kwargs)

    @classmethod
    def parse_and_create(cls, **kwargs):
        content = cls.parse_template(**kwargs)

        try:
            is_dry = kwargs['dry']
        except KeyError:
            is_dry = False

        if is_dry:
            cls.log_dry(**kwargs, content=content)
        else:
            cls.create_file(
                path=kwargs['path'],
                filename=kwargs['filename'],
                file_content=content
            )

    @classmethod
    def remove_import(cls, **kwargs):
        try:
            is_dry = kwargs['dry']
        except KeyError:
            is_dry = False

        if not is_dry:
            filename = '__init__.py'

            path = kwargs['path']

            template = kwargs['template']

            try:
                content = template.render(**kwargs)
            except Exception:
                raise EnvironmentError

            try:
                os.chdir(path)

                # Find import line to delete
                s = open(filename).read()
                s = s.replace(content, '')
                f = open(filename, 'w')
                f.write(s)
                f.close()

                for line in fileinput.FileInput(filename, inplace=1):
                    if line.rstrip():
                        print(line, end='')

            except FileNotFoundError:
                raise click.Abort

    @classmethod
    def show_files(cls, path):
        os.listdir(path)



from .base_run_helper import BaseRunHelper
from .build_helper import BuildHelper
from .docker_helper import DockerHelper
from .environment_helper import EnvironmentHelper
from .migration_helper import MigrationHelper
from .server_helper import ServerHelper
from .fixture_helper import FixtureHelper



import os
import subprocess
from .base_run_helper import BaseRunHelper

DEFAULT_RUNSERVER_COMMAND = ['python3', 'manage.py', 'runserver', '8000']

DEFAULT_PROCESS_SUBGROUP = 'django-run-server-group-soliloquy'


class ServerHelper(BaseRunHelper):

    @classmethod
    def run(cls, **kwargs):
        """
        Starts the default Django development server.
        """
        os.chdir(kwargs['path'])

        try:
            DEFAULT_RUNSERVER_COMMAND[-1] = str(kwargs['port']) if kwargs['port'] else DEFAULT_RUNSERVER_COMMAND[-1]
        except KeyError:
            pass

        subprocess.call(DEFAULT_RUNSERVER_COMMAND)



import re
from .base_run_helper import BaseRunHelper


class EnvironmentHelper(BaseRunHelper):

    @staticmethod
    def __read_environment(env_file):
        """
        Use this to read environment variables from a .env file

        Based on implementations found on the links below:
            https://gist.github.com/bennylope/2999704
            https://wellfire.co/learn/easier-12-factor-django/
        """

        try:
            with open(env_file) as f:
                content = f.read()
        except IOError:
            content = ''

        keys = {}

        for line in content.splitlines():
            first_match = re.match(r'\A([A-Za-z_0-9]+)=(.*)\Z', line)
            if first_match:
                key, value = first_match.group(1), first_match.group(2)
                second_match = re.match(r"\A'(.*)'\Z", value)
                if second_match:
                    value = second_match.group(1)
                third_match = re.match(r'\A"(.*)"\Z', value)
                if third_match:
                    value = re.sub(r'\\(.)', r'\1', third_match.group(1))
                keys[key] = value

        return keys

    def __copy_env(self, env_file, project_name=None, dokku=True):
        keys = self.__read_environment(env_file)

        options = ['.env-dokku', '.env-example']

        output = options[0] if dokku else options[1]

        with open(output, 'w') as file:
            for k, v in keys.items():
                if dokku:
                    v = v.split("#")[0]
                    if v not in ['False', 'True']:
                        v = f"'{v}'"
                    statement = f"dokku config:set --no-restart {project_name} {k}={v}\n"
                else:
                    statement = f'{k}=\n'
                file.write(statement)

    def copy_dokku(self, env_file, project_name):
        self.__copy_env(env_file, project_name)

    def copy_example(self, env_file):
        self.__copy_env(env_file, dokku=False)

    def run(self, **kwargs):
        project_name = kwargs['project_name']

        env_file = kwargs['filepath'] if kwargs['filepath'] else kwargs['path'] + '/.env'

        self.copy_example(env_file)
        self.copy_dokku(env_file, project_name)



# django-clite
from __future__ import unicode_literals
import click
import inflection
import os


def file_created(filename=''):
    log_success("Successfully created %s" % filename)


def file_exists(filename=''):
    log_error("File already exists %s" % filename)


def find_management_file(cwd):
    """
    Searches the current working directory for any of 4 key Django files:
    manage.py, settings.py, wsgi.py, and apps.py to determine from where a command
    is being run. This is used to ensure commands that create or delete resources
    are always run in the correct directory and within a Django project.
    """
    code = 0
    path = None
    management = None
    file = None

    levels = (
        ('manage.py', 1),    # <-- project
        ('settings.py', 2),  # <-- project/project
        ('wsgi.py', 2),      # <-- project/project
        ('apps.py', 3),      # <-- project/project/app
    )

    for filename, c in levels:
        if filename in os.listdir(cwd):
            code = c
            path = os.getcwd()

    if code == 1:
        management = path
        path = f"{path}/{path.split('/')[-1]}"
        file = management + '/manage.py'
    elif code == 2:
        management = path.rsplit('/', 1)[0]
        file = management + '/manage.py'
    elif code == 3:
        management = path.rsplit('/', 2)[0]
        path = path.rsplit('/', 1)[0]
        file = management + '/manage.py'
    return path, management, code, file


def get_project_name(management_file, find_first=False):
    if find_first:
        p, m, c, f = find_management_file('.')
        management_file = f

    with open(management_file, 'r') as file:
        for line in file:
            if 'DJANGO_SETTINGS_MODULE' in line:
                line = line.\
                    replace("'", "").\
                    replace(" ", "").\
                    split(',')[-1].\
                    split('.')[0]
                return line
    return None


def log_info(message):
    click.secho(message, fg='yellow')


def log_success(message):
    click.secho(message, fg='green')


def log_error(message):
    click.secho(message, fg='red')


def sanitized_string(text):
    """
    Ensures strings are properly sanitized and
    no special characters are present.
    """
    r = inflection.transliterate(text)
    r = r.replace(' ', '-')
    return inflection.underscore(r).lower()



from jinja2 import Template

router_template = Template("""from rest_framework import routers

'''
Your router endpoints are automatically added to the urlpatterns in urls.py for this app.
To make your app's urls available to the entire project, include your app urls in the urlpatterns for your project:

In your project's urls.py file:

from django.urls import include, path
urlpatterns = [
   # Other paths like admin and login stuff...
   path('{{ app_name }}', include('{{ project_name }}.{{app_name}}.urls'))
]

'''

router = routers.SimpleRouter(trailing_slash=False)
""")



from jinja2 import Template


app_urls_template = Template("""\"""
urlpatterns for {{ project }}.{{ app_name }}
\"""

from django.urls import include, path
from .viewsets.router import router


urlpatterns = [
   path('', include(router.urls)),
]
""")



from jinja2 import Template


app_template = Template("""{
  "scripts": {
    "dokku": {
      "predeploy": "/code/manage.py migrate --noinput",
      "postdeploy": "/code/./notify.sh"
    }
  }
}
""")


dokku_checks_template = Template("""DOKKU_CHECKS_WAIT={{ wait }}
DOKKU_CHECKS_TIMEOUT={{ timeout }}
""")


dokku_scale_template = Template("""web={{ web }}""")


procfile_template = Template("""release: ./release.sh
web: gunicorn {{ project }}.wsgi
""")



from jinja2 import Template


page_template = Template("""{{ '{% load static from staticfiles %}' }}
{{ '{% load cache %}' }}

{{ '{% comment %}' }}
    Template for {{ name }}
    Describe the template here.
{{ '{% endcomment %}' }}

{{ '{% block header %}{% endblock header %}' }}

{{ '{% block body %}{% endblock body %}' }}

{{ '{% block footer %}{% endblock footer %}' }}

{{ '{% block scripts %}{% endblock scripts %}' }}
""")



from django_clite.cli import log_success
from django_clite.cli.commands.base_helper import BaseHelper
from django_clite.cli.templates.template import page_template


class TemplateHelper(BaseHelper):

    def create(self, **kwargs):
        filename = f"{kwargs['name'].lower()}.html"

        template = page_template

        path = kwargs['path']

        self.parse_and_create(
            filename=filename,
            name=kwargs['name'],
            path=path,
            template=template,
            dry=kwargs['dry']
        )

        log_success("Successfully created HTML template.")

    def delete(self, **kwargs):
        model = self.check_noun(kwargs['model'])

        filename = f"{model.lower()}.html"

        if self.destroy(filename=filename, **kwargs):

            log_success('Successfully deleted HTML template.')

# end class



import click
import os
from .helper import CreatorHelper
from django_clite.cli import (
    log_error,
    log_info,
    find_management_file,
    sanitized_string
)

DEFAULT_MANAGEMENT_ERROR = """The CLI could not decipher your django project.
Please try to create your app inside a Django project directory.
"""

DEFAULT_MANAGEMENT_TIP = """Tip:
If your project is called `website`, for example, 
you need to run `D new app app_name` from within `website` or `website/website`.
"""


def wrong_place_warning(ctx):
    if ctx.obj['path']:
        return False
    return True



@click.group()
@click.pass_context
@click.option('--dry', is_flag=True, help="Display output without creating files.")
def create(ctx, dry):
    """
    Creates projects and apps.
    """
    ctx.ensure_object(dict)
    ctx.obj['dry'] = dry
    ctx.obj['cwd'] = os.getcwd()
    ctx.obj['path'], ctx.obj['management'], ctx.obj['code'] = find_management_file(os.getcwd())


@create.command()
@click.argument('name')
@click.option('--docker', is_flag=True, help="Add support for Docker.")
@click.option('--dokku', is_flag=True, help="Add support for Dokku.")
@click.option('-a', '--custom-auth', is_flag=True, help="Add support for custom AUTH_USER_MODEL.")
@click.argument('apps', nargs=-1)
@click.pass_context
def project(ctx, name, docker, dokku, custom_auth, apps):
    """
    Creates a new django project.

    This is similar to using `django-admin startproject project_name` but with some added functionality.
    The command can handle the creation of apps upon creation of any project, you can simply specify the name of
    the apps after the project name:

        D new project website app1 app2 app3 ...

    Another added functionality is the ability to setup Docker and Dokku.
    To do so, use the respective flags --docker and --dokku.

    Also helpful is the ability to initialize your application with a custom authentication model of your choosing.
    When the --custom-auth flag is specified, CLI will add an `authentication` app to your project
    along with a subclass of the AbstractUser model which you can extend for your own authentication purposes.
    """
    helper = CreatorHelper()

    if not ctx.obj['dry']:
        helper.create_project(
            project=name,
            apps=apps,
            docker=docker,
            dokku=dokku,
            dry=ctx.obj['dry']
        )
        if custom_auth:
            os.chdir(sanitized_string(name))
            ctx.obj['path'] = os.getcwd()
            ctx.invoke(app, apps=['authentication'], custom_auth=True)


@create.command()
@click.argument('apps', nargs=-1)
@click.option('--custom-auth', is_flag=True, help="Add User for custom authentication.")
@click.pass_context
def app(ctx, apps, custom_auth):
    """
    Creates new django apps.

    This is similar to using `django-admin startapp app_name`
    but adds more structure to your app by creating packages for forms, models,
    serializers, tests, templates, views, and viewsets.

    The command can accept multiple apps as arguments. So,

        D new app shop blog forum

    will work prompt the CLI to create all 4 apps two levels within your project's
    directory, i.e mysite/mysite. The CLI tries to identify where the management module for your
    project is so it can place your app files in the correct location. This helps with consistency
    as the CLI can infer module/package scopes when performing other automatic configurations.

    As part of the CLI convention, each app is assigned its own `urls.py` file, which can be used to route urls on a per app basis. Another convention the CLI adopts is
    to add a viewsets package to the app's directory by default (for use with DRF). Within the viewsets directory
    a DRF router is instantiated in `router.py` and its urls added to each app's urlpatterns by default.
    """
    helper = CreatorHelper()

    will_create_app = True

    if not ctx.obj['path']:
        log_error(DEFAULT_MANAGEMENT_ERROR)
        log_info(DEFAULT_MANAGEMENT_TIP)
        will_create_app = False

    if will_create_app:

        if not ctx.obj['dry']:
            path = ctx.obj['path']
            project_name = ctx.obj['path'].rsplit('/')[-1]

            os.chdir(path)
            for app_name in apps:
                helper.create_app(
                    app_name=app_name,
                    project=project_name,
                    dry=ctx.obj['dry'],
                    custom_auth=custom_auth
                )



from jinja2 import Template


route_view = Template("""
def {{ route.lower() }}(request):
    context = {
        'active': True,
        'route': '{{ route.lower() }}',
     }
    return render(request, '{{ route.lower() }}.html', context)
""")

route__simple_template = Template("""
<!DOCTYPE html>
<html lang="en">
<head>
    <title>{{ route.capitalize() }}</title>
    {% if framework %}<link rel="stylesheet" href="{{ framework }}/css/{{ framework }}.css">
    <script src="{{ framework }}/js/{{ framework }}.js"></script>{% endif %}
    <link rel="stylesheet" href="static/css/main.css">
</head>
<body>
    <h1>Template for {{ route.capitalize() }}</h1>

    <script src="static/js/main.js"></script>
</body>
</html>
""")

route_template_with_framework = Template("""
<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width, initial-scale=1">
    <title>{{ route.capitalize() }}</title>
    <script src="https://code.jquery.com/jquery-3.3.1.slim.min.js" integrity="sha384-q8i/X+965DzO0rT7abK41JStQIAqVgRVzpbzo5smXKp4YfRvH+8abtTE1Pi6jizo" crossorigin="anonymous"></script>
    {% if framework %}<link rel="stylesheet" href="{{ framework[0]['link'] }}" integrity="{{ framework[0]['integrity']}}" crossorigin="anonymous">
    <script src="{{ framework[1]['src'] }}" integrity="{{ framework[0]['integrity']}}" crossorigin="anonymous"></script>{% endif %}
    <link rel="stylesheet" href="static/css/main.css">
</head>
<body>
    <h1>Template for {{ route.capitalize() }}</h1>

    <script src="static/js/main.js"></script>
</body>
</html>
""")

route_url = Template("""
urlpatterns += [
    path('{% if route == 'index' %}/{% else %}{{ route }}/{% endif %}', {{ route }}, name='{{ route }}'),
]
""")

route_path = Template("""
path('{% if route == 'index' %}/{% else %}{{ route }}/{% endif %}', {{ route }}, name='{{ route }}'),
""")



from setuptools import find_packages, setup


with open("README.md", "r") as f:
    long_description = f.read()


setup(
    name="django-clite",
    version="0.1.0-beta.5",
    author="Leo Neto",
    author_email="leo@ekletik.com",
    url="https://bitbucket.org/oleoneto/django-clite",
    description="CLI for managing Django projects",
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords="django automate cli command line tools rails ember python framework",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django :: 2.2',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    license="BSD",
    install_requires=[
        'Click==7.0',
        'jinja2',
        'inflection',
        'django>=2.2.4',
        'tox'
    ],
    zip_safe=False,
    include_package_data=True,
    packages=find_packages(),
    project_urls={
        "Documentation": "https://bitbucket.org/oleoneto/django-clite/",
        "Source Code": "https://bitbucket.org/oleoneto/django-clite/",
    },
    entry_points={
        'console_scripts': [
            'D=app:main',
            'django-clite=app:main',
        ]
    },
    scripts=['app.py']
)



from jinja2 import Template


viewset_template = Template(
    """from rest_framework import viewsets
from rest_framework import permissions
from .router import router
from ..models import {{ model.capitalize() }}
from ..serializers import {{ model.capitalize() }}Serializer


class {{ model.capitalize() }}ViewSet(viewsets.{% if read_only %}ReadOnlyModelViewSet{% else %}ModelViewSet{% endif %}):
    queryset = {{ model.capitalize() }}.objects.all()
    serializer_class = {{ model.capitalize() }}Serializer
    permission_classes = [permissions.IsAuthenticatedOrReadOnly]


router.register('{{ route.lower() }}', {{ model.capitalize() }}ViewSet)
""")

viewset_import_template = Template("""from .{{ model.lower() }} import {{ model.capitalize() }}ViewSet""")



from django_clite.cli import log_success
from django_clite.cli.commands.base_helper import BaseHelper
from django_clite.cli.templates.test import test_case_template, test_case_import_template


class TestHelper(BaseHelper):

    def create(self, **kwargs):
        model = self.check_noun(kwargs['model'])

        path = kwargs['path']

        filename = f"{model}.py"

        self.parse_and_create(
            model=model,
            filename=filename,
            template=test_case_template,
            path=path,
            dry=kwargs['dry']
        )

        self.add_import(
            model=model,
            template=test_case_import_template,
            path=path
        )

        log_success("Successfully created TestCase.")

    def delete(self, **kwargs):
        model = self.check_noun(kwargs['model'])

        filename = f"{model.lower()}.py"

        template = test_case_import_template

        if self.destroy(filename=filename, **kwargs):

            self.remove_import(template=template, **kwargs)

            log_success('Successfully deleted TestCase.')

    @classmethod
    def create_auth_user(cls, **kwargs):
        kwargs['model'] = 'User'

        kwargs['filename'] = 'user.py'

        kwargs['path'] = f"{kwargs['path']}/tests/"

        cls.parse_and_create(
            model=kwargs['model'],
            filename=kwargs['filename'],
            project_name=kwargs['project'],
            template=test_case_template,
            path=kwargs['path']
        )

        cls.add_import(**kwargs, template=test_case_import_template)

        log_success("Successfully created TestCase.")



import click
import os
from django_clite.cli import log_error
from .helpers import (
    AdminHelper,
    FormHelper,
    ManagerHelper,
    ModelHelper,
    SerializerHelper,
    TemplateHelper,
    TestHelper,
    ViewHelper,
    ViewSetHelper,
    SQLHelper
)


def not_an_app_directory_warning():
    if not ('apps.py' in os.listdir('.')):
        log_error("Not inside an app directory")
        raise click.Abort


@click.group()
@click.option('--dry', is_flag=True, help="Display output without creating files.")
@click.pass_context
def generate(ctx, dry):
    """
    Adds models, routes, and other resources
    """
    not_an_app_directory_warning()

    ctx.ensure_object(dict)
    ctx.obj['dry'] = dry
    ctx.obj['in_app'] = 'apps.py' in os.listdir('.')
    ctx.obj['cwd'] = os.getcwd()
    ctx.obj['admin'] = f"{os.getcwd()}/admin/"
    ctx.obj['admin_inlines'] = f"{os.getcwd()}/admin/inlines/"
    ctx.obj['forms'] = f"{os.getcwd()}/forms/"
    ctx.obj['models'] = f"{os.getcwd()}/models/"
    ctx.obj['managers'] = f"{os.getcwd()}/models/managers"
    ctx.obj['serializers'] = f"{os.getcwd()}/serializers/"
    ctx.obj['tests'] = f"{os.getcwd()}/tests/"
    ctx.obj['templates'] = f"{os.getcwd()}/templates/"
    ctx.obj['views'] = f"{os.getcwd()}/views/"
    ctx.obj['viewsets'] = f"{os.getcwd()}/viewsets/"


@generate.command()
@click.argument('name')
@click.option('--inline', is_flag=True, help='Register admin model as inline.')
@click.pass_context
def admin(ctx, name, inline):
    """
    Generates an admin model within the admin package.
    """

    path = ctx.obj['admin']

    if inline:
        path = ctx.obj['admin_inlines']

    AdminHelper().create(model=name, inline=inline, path=path, dry=ctx.obj['dry'])


@generate.command()
@click.argument("name", required=True)
@click.pass_context
def form(ctx, name):
    """
    Generates a model form within the forms package.
    """

    path = ctx.obj['forms']

    FormHelper().create(model=name, path=path, dry=ctx.obj['dry'])


@generate.command()
@click.argument("name", required=True)
@click.pass_context
def manager(ctx, name):
    """
    Generates a model manager under the model managers directory.
    """
    path = ctx.obj['managers']

    ManagerHelper().create(
        model=name,
        path=path,
        dry=ctx.obj['dry']
    )


@generate.command()
@click.option('-a', '--abstract', is_flag=True, help="Creates an abstract model type.")
@click.option('-t', '--test-case', is_flag=True, help="Creates a TestCase for model.")
@click.option('-v', '--view', is_flag=True, help="Make model an SQL view.")
@click.option('-f', '--full', is_flag=True, help="Adds all related resources and TestCase")
@click.option('--register-admin', is_flag=True, help="Register model to admin site.")
@click.option('--register-inline', is_flag=True, help="Register model to admin site as inline.")
@click.option('-i', '--inherits', required=False, help="Add model inheritance.")
@click.argument("name", required=True)
@click.argument("fields", nargs=-1, required=False)
@click.pass_context
def model(ctx, name, full, abstract, fields, register_admin, register_inline, test_case, inherits, view):
    """
    Generates a model under the models directory.
    One can specify multiple attributes after the model's name, like so:

        D g model track int:number char:title fk:album bool:is_favorite

    This will generate a Track model and add a foreign key of Album.
    If the model is to be added to admin.site one can optionally opt in by specifying the --register-admin flag.
    """

    path = ctx.obj['models']

    ModelHelper().create(
        model=name,
        abstract=abstract,
        fields=fields,
        path=path,
        dry=ctx.obj['dry'],
        inherits=inherits,
        view=view
    )

    if register_admin or full:
        ctx.invoke(admin, name=name)

    if register_inline or full:
        ctx.invoke(admin, name=name, inline=True)

    if test_case or full:
        ctx.invoke(test, model=name)

    if full:
        ctx.invoke(form, name=name)
        ctx.invoke(serializer, name=name)
        ctx.invoke(template, name=name)
        ctx.invoke(view, name=name, list=True)
        ctx.invoke(view, name=name, detail=True)
        ctx.invoke(viewset, name=name)


@generate.command()
@click.argument("name", required=True)
@click.argument("fields", nargs=-1)
@click.option('-i', '--inherits', required=False, help="Add model inheritance.")
@click.option('--api', is_flag=True, help='Only add api-related files.')
@click.pass_context
def resource(ctx, name, fields, inherits, api):
    """
    Generates an app resource.

    This is ideal to add a model along with admin, serializer, view, viewset, template, and tests.
    You can invoke this command the same way you would the model command:

        D g resource track int:number char:title fk:album bool:is_featured

    This will generate a model with the specified attributes and all the related modules specified above.

    In case you're building an api, and don't need forms, templates and views, you can pass the --api flag to the command
    in order to prevent these files from being created.
    """

    ctx.invoke(
        model,
        name=name,
        register_admin=True,
        register_inline=True,
        fields=fields,
        test_case=True,
        inherits=inherits
    )

    ctx.invoke(serializer, name=name)

    ctx.invoke(viewset, name=name)

    if not api:
        ctx.invoke(form, name=name)
        ctx.invoke(template, name=name)
        ctx.invoke(view, name=name, list=True)


@generate.command()
@click.argument("name", required=True)
@click.pass_context
def serializer(ctx, name):
    """
    Generates a serializer for a given model.

    Checks for the existence of the specified model in models.py
    before attempting to create a serializer for it. Aborts if model is not found.
    """

    path = ctx.obj['serializers']

    SerializerHelper().create(model=name, path=path, dry=ctx.obj['dry'])


@generate.command()
@click.argument("name", required=True)
@click.pass_context
def template(ctx, name):
    """
    Generates an html template.
    """

    path = ctx.obj['templates']

    TemplateHelper().create(name=name, path=path, dry=ctx.obj['dry'])


@generate.command()
@click.argument("model", required=True)
@click.pass_context
def test(ctx, model):
    """
    Generates a new TestCase.
    """

    path = ctx.obj['tests']

    TestHelper().create(
        model=model,
        path=path,
        dry=ctx.obj['dry']
    )


@generate.command()
@click.argument("name", required=True)
@click.option('-l', '--list', is_flag=True, help="Create model list view.")
@click.option('-d', '--detail', is_flag=True, help="Create model detail view.")
@click.option('-t', is_flag=True, help='Generate related template.')
@click.pass_context
def view(ctx, name, list, detail, t):
    """
    Generates a view function or class.
    """

    path = ctx.obj['views']

    ViewHelper().create(
        model=name,
        name=name,
        detail=detail,
        list=list,
        path=path,
        dry=ctx.obj['dry']
    )

    if t:
        ctx.invoke(template, name=name)


@generate.command()
@click.option('-r', '--read-only', is_flag=True, help="Create a read-only viewset.")
@click.argument("name", required=True)
@click.pass_context
def viewset(ctx, read_only, name):
    """
    Generates a viewset for a serializable model.
    """

    path = ctx.obj['viewsets']

    ViewSetHelper().create(model=name, path=path, read_only=read_only, dry=ctx.obj['dry'])


# @generate.command()
@click.option('-s', '--source', is_flag=True, help="Base table for SQL view.")
@click.argument("name", required=True)
@click.argument("fields", nargs=-1, required=False)
@click.pass_context
def sql_view(ctx, name, fields, source):
    """
    Generates a model as an SQL view.
    One can specify multiple attributes after the model's name, like so:

        D g sql_view track int:number char:title --source app_tracks
    """

    path = ctx.obj['models']

    SQLHelper().create(
        model=name,
        fields=fields,
        path=path,
        dry=ctx.obj['dry'],
        table=source
    )



from jinja2 import Template


default_function_view_template = Template(
    """from django.shortcuts import HttpResponse
from django.shortcuts import render
from django.view.decorators.cache import cache_page
import datetime

\"""
Enable caching if needed. This page will be cached fro 15 minutes.
@cache_page(60 * 15)
\"""
def {{ name.lower() }}_view(request):
    template = "{{ name.lower() }}.html"
    now = datetime.datetime.now()
    context = {
        'date': now
    }

    \"""
    Alternatively, your view can return HTML directly like so:
    html = "<html><body><h1>{{ name.capitalize() }}View</h1>It is now %s.</body></html>" % now
    return HttpResponse(html)
    \"""
    return render(request, template, context)
""")

default_class_view_template = Template("""from django.utils import timezone{% if list %}
from django.views.generic.list import ListView
from ..models import {{ model.capitalize() }}


class {{ model.capitalize() }}ListView(ListView):
{% else %}
from django.view.generic.detail import DetailView
from ..models import {{ model.capitalize() }}


class {{ model.capitalize() }}DetailView(DetailView):
{% endif %}
    model = {{ model.capitalize() }}
    {% if list %}paginate_by = 20\n{% endif %}
    def get_context_data(self, **kwargs):
        context = super().get_context_data(**kwargs)
        context['now'] = timezone.now()
        return context
""")

function_view_import_template = Template(
    """from .{{ model.lower() }} import {{ model.lower() }}"""
)

default_function_view_import_template = Template(
    """from .{{ name.lower() }} import {{ name.lower() }}_view"""
)

default_class_view_import_template = Template(
    """from .{{ model.lower() }} import {{ model.capitalize() }}{% if list %}ListView{% elif detail %}DetailView{% endif %}"""
)



import click
import os
from django_clite.cli import log_error
from django_clite.cli.commands.generator.helpers import (
    AdminHelper,
    FormHelper,
    ManagerHelper,
    ModelHelper,
    SerializerHelper,
    TemplateHelper,
    TestHelper,
    ViewHelper,
    ViewSetHelper
)


def not_an_app_directory_warning():
    if not ('apps.py' in os.listdir('.')):
        log_error("Not inside an app directory")
        raise click.Abort


def confirm_delete():
    if not click.confirm("Are you sure you want to delete this file?", show_default=True, default=False):
        raise click.Abort()
    return True


@click.group()
@click.option('--dry', '--dry-run', is_flag=True, help="Display output without deleting files")
@click.pass_context
def destroy(ctx, dry):
    """
    Removes models, serializers, and other resources
    """
    if not dry:
        not_an_app_directory_warning()

    ctx.ensure_object(dict)
    ctx.obj['dry'] = dry
    ctx.obj['in_app'] = 'apps.py' in os.listdir('.')
    ctx.obj['cwd'] = os.getcwd()
    ctx.obj['admin'] = f"{os.getcwd()}/admin/"
    ctx.obj['admin_inlines'] = f"{os.getcwd()}/admin/inlines/"
    ctx.obj['forms'] = f"{os.getcwd()}/forms/"
    ctx.obj['models'] = f"{os.getcwd()}/models/"
    ctx.obj['managers'] = f"{os.getcwd()}/models/managers/"
    ctx.obj['serializers'] = f"{os.getcwd()}/serializers/"
    ctx.obj['tests'] = f"{os.getcwd()}/tests/"
    ctx.obj['templates'] = f"{os.getcwd()}/templates/"
    ctx.obj['views'] = f"{os.getcwd()}/views/"
    ctx.obj['viewsets'] = f"{os.getcwd()}/viewsets/"
    ctx.obj['confirm'] = confirm_delete()


@destroy.command()
@click.argument('name')
@click.option('--inline', is_flag=True, help='Destroy inline admin model')
@click.pass_context
def admin(ctx, name, inline):
    """
    Destroys an admin model or inline.
    """

    # Default helper
    helper = AdminHelper()

    path = ctx.obj['admin']

    if inline:
        path = ctx.obj['admin_inlines']

    if ctx.obj['confirm']:
        helper.delete(
            model=name,
            path=path,
            inline=inline,
            dry=ctx.obj['dry']
        )


@destroy.command()
@click.argument('name', required=True)
@click.pass_context
def form(ctx, name):
    """
    Destroys a form.
    """

    path = ctx.obj['forms']

    # Default helper
    helper = FormHelper()

    if ctx.obj['confirm']:
        helper.delete(
            path=path,
            model=name,
            dry=ctx.obj['dry']
        )


@destroy.command()
@click.argument('name', required=True)
@click.pass_context
def manager(ctx, name):
    """
    Destroys a model manager.
    """

    path = ctx.obj['managers']

    ManagerHelper.delete(
        path=path,
        model=name,
        dry=ctx.obj['dry']
    )


@destroy.command()
@click.argument('name')
@click.option('--unregister-admin', is_flag=True, help="Unregister model from the admin site.")
@click.option('--unregister-inline', is_flag=True, help="Unregister inline model from the admin site.")
@click.option('--test-case', is_flag=True, help="Delete TestCases for model.")
@click.option('--full', is_flag=True, help="Delete admin, inline, and TestCase")
@click.pass_context
def model(ctx, name, full, unregister_admin, unregister_inline, test_case):
    """
    Destroys a model.
    """

    path = ctx.obj['models']

    # Default helper
    helper = ModelHelper()

    if ctx.obj['confirm']:
        helper.delete(
            model=name,
            path=path,
            dry=ctx.obj['dry']
        )

        if unregister_admin or full:
            ctx.invoke(admin, name=name)

        if unregister_inline or full:
            ctx.invoke(admin, name=name, inline=True)

        if test_case or full:
            ctx.invoke(test, name=name)

        if full:
            ctx.invoke(form, name=name)
            ctx.invoke(serializer, name=name)
            ctx.invoke(template, name=name)
            ctx.invoke(view, name=name, list=True)
            ctx.invoke(view, name=name, detail=True)
            ctx.invoke(viewset, name=name)


@destroy.command()
@click.argument('name')
@click.pass_context
def resource(ctx, name):
    """
    Destroys a resource.
    """

    ctx.invoke(
        model,
        name=name,
        unregister_admin=True,
        unregister_inline=True,
        test_case=True
    )

    ctx.invoke(serializer, name=name)

    ctx.invoke(viewset, name=name)

    ctx.invoke(form, name=name)

    ctx.invoke(template, name=name)

    ctx.invoke(view, name=name)


@destroy.command()
@click.argument('name')
@click.pass_context
def serializer(ctx, name):
    """
    Destroys a serializer.
    """

    path = ctx.obj['serializers']

    # Default helper
    helper = SerializerHelper()

    if ctx.obj['confirm']:
        helper.delete(
            path=path,
            model=name,
            dry=ctx.obj['dry']
        )


@destroy.command()
@click.argument('name')
@click.option('-l', '--list', is_flag=True, help='Deletes a model list view')
@click.option('-d', '--detail', is_flag=True, help='Deletes a model detail view')
@click.pass_context
def view(ctx, name, list, detail):
    """
    Destroys a view.
    """

    path = ctx.obj['views']

    # Default helper
    helper = ViewHelper()

    if ctx.obj['confirm']:
        helper.delete(
            path=path,
            model=name,
            name=name,
            list=list,
            detail=detail,
            dry=ctx.obj['dry']
        )


@destroy.command()
@click.argument('name')
@click.pass_context
def viewset(ctx, name):
    """
    Destroys a viewset.
    """

    path = ctx.obj['viewsets']

    # Default helper
    helper = ViewSetHelper()

    if ctx.obj['confirm']:
        helper.delete(
            path=path,
            model=name,
            dry=ctx.obj['dry']
        )


@destroy.command()
@click.argument('name')
@click.pass_context
def template(ctx, name):
    """
    Destroys a template.
    """

    path = ctx.obj['templates']

    # Default helper
    helper = TemplateHelper()

    if ctx.obj['confirm']:
        helper.delete(
            path=path,
            model=name,
            dry=ctx.obj['dry']
        )


@destroy.command()
@click.argument('name')
@click.pass_context
def test(ctx, name):
    """
    Destroys a TestCase.
    """

    # Default helper
    helper = TestHelper()

    path = ctx.obj['tests']

    if ctx.obj['confirm']:
        helper.delete(
            model=name,
            path=path,
            dry=ctx.obj['dry']
        )
